/****************************************************************************
	Xiliinx SpartanII JTAG Controll Functions

	Copyright(c) 2003 Tatsuyuki Satoh

	note:

		use MINW32 to compile.

	Histry

	2002. 5.29 Ver.1.14 Separated from spcisp.c

****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "bscan.h"      /* JTAG BounderyScan Access Controll */

/* -------------------- error status -------------------- */
#define ERROR_PROBE  0x4000
#define ERROR_TARGET 0x2000

/* -------------------- BoundaryScan(JTAG) Configration -------------------- */
/*
	Xilinx XAPP188 (v2.0)
*/

/* StartanII JTAG Instruction code */
#define EXTEST   0
#define SAMPLE   1
#define USER1    2
#define USER2    3
#define CFG_OUT  4
#define CFG_IN   5
#define INTEST   7
#define USERCODE 8
#define IDCODE   9
#define HIGHZ    10
#define JSTART   12
#define BYPASS   31

/* StartanII JTAG Configration Command Code */
#define CMD_WCFG   0x01 /* 0001 write config   */
#define CMD_LFRM   0x01 /* 0011 last frame     */
#define CMD_RCFG   0x04 /* 1110 read config    */
#define CMD_START  0x05 /* 1110 startup        */
#define CMD_RCAP   0x06 /* 1110 reset capture  */
#define CMD_RCRC   0x07 /* 1110 reset CRC      */
#define CMD_AHIGH  0x08 /* 1110 GHIGH_B assert */
#define CMD_SWITCH 0x09 /* 1110 SWITCH         */

/****************************************************************************/
/* swap bits in BYTE data                                                   */
/****************************************************************************/
static BYTE swap_byte(BYTE value)
{
	BYTE ret = 0;

	if(value&0x01) ret |= 0x80;
	if(value&0x02) ret |= 0x40;
	if(value&0x04) ret |= 0x20;
	if(value&0x08) ret |= 0x10;
	if(value&0x10) ret |= 0x08;
	if(value&0x20) ret |= 0x04;
	if(value&0x40) ret |= 0x02;
	if(value&0x80) ret |= 0x01;

	return ret;
}


// get rxdata to DWORD conversion (bigendian)
static DWORD get_dword(BYTE *data)
{
	return (DWORD)data[0]*0x1000000 |
		   (DWORD)data[1]*0x10000   |
		   (DWORD)data[2]*0x100     |
		   (DWORD)data[3];
}

// swap bits in DWORD data
static DWORD swap_dword(DWORD value)
{
	DWORD ret = 0;
	int b;
	
	for(b=0;b<32;b++)
	{
		if( value & (1<<b) )
			ret |= 0x80000000>>b;
	}
	return ret;
}

/***************************************************************************/
/* get device id                                                           */
/***************************************************************************/
DWORD spa2_read_deviceid(void)
{
	BYTE obuf[4];

	// IDCODE read & check
	bscan_reset(6);
	bscan_inst(IDCODE,NULL,obuf,32,1);

	return get_dword(obuf);
}

/***************************************************************************/
/* get status register                                                    */
/***************************************************************************/
DWORD spa2_read_status(void)
{
	BYTE obuf[4];
	DWORD sts;
	static const BYTE sts_data[] = {
		0x28,0x00,0xe0,0x01,
		0x00,0x00,0x00,0x00 };

	// read status
	bscan_reset(6);
	bscan_before_dr = (32-bscan_tdi_bypass)&31;
	bscan_inst(CFG_IN,sts_data,NULL,sizeof(sts_data)*8,1);
	bscan_before_dr = bscan_tdi_bypass;
	bscan_inst(CFG_OUT,NULL,NULL,1,0); // dummy 1 bit ?
	bscan_dr(NULL,obuf,32,1);

	return get_dword(obuf);
}

/***************************************************************************/
/* SpartanII shutdown                                                      */
/***************************************************************************/
DWORD spa2_shutdown(void)
{
	static const BYTE sht_data0[] =
{
	0x30,0x00,0x80,0x01, // 1.CMD write
	0x00,0x00,0x00,CMD_SWITCH,
	0x30,0x01,0x20,0x01, // 2.COR write
	0x00,0xa0,0xbf,0x2d, //       SHUTDOWN = 1
//	0x00,0x80,0xbf,0x2d, //       SHUTDOWN = 1 (err)

	0x30,0x00,0x80,0x01, // 3.CMD write
	0x00,0x00,0x00,CMD_START,
	0x30,0x00,0x80,0x01, // 4.CMD write
	0x00,0x00,0x00,CMD_RCRC,
	0x00,0x00,0x00,0x00}; // flush PIPE

	static const BYTE sht_data1[] =
{
	0x30,0x00,0x80,0x01, // 4.CMD AHIGH
	0x00,0x00,0x00,CMD_AHIGH,
	0x30,0x01,0x20,0x01, // 2.COR SHUTDOWN = 0
	0x00,0x80,0x3f,0x2d, //       SHUTDOWN = 0
	0x00,0x00,0x00,0x00}; // flush PIPE

	bscan_reset(6);
	bscan_before_dr = (32-bscan_tdi_bypass)&31;
	bscan_inst(CFG_IN,sht_data0,NULL,sizeof(sht_data0)*8,1);	//   SHUTDOWN
	sleep(50);
	bscan_inst(JSTART,NULL,NULL,16,1);							// 5. 6. TCK 12 cycle
	sleep(50);
	bscan_inst(CFG_IN,sht_data1,NULL,sizeof(sht_data1)*8,1);	// 7. GHIGH command
	sleep(50);
	bscan_before_dr = bscan_tdi_bypass;

	return 0;
}

/***************************************************************************/
/* CONFIG_START() : BoundaryScan start Configration                        */
/***************************************************************************/
DWORD spa2_enter_config(void)
{
	// Enter CFG_IN phase
	bscan_reset(6);
	bscan_before_dr = (32-bscan_tdi_bypass)&31; // 32 orign
	bscan_inst(CFG_IN,NULL,NULL,0,0);
	bscan_before_dr = bscan_tdi_bypass;

	return 0;
}

/***************************************************************************/
/* spa2_write_config : BoundaryScan write Configration data                */
/***************************************************************************/
DWORD spa2_write_config(BYTE *data,DWORD size,int last)
{
	//phase 9,10,11
	bscan_dr(data,NULL,size*8,last);

	//ŌH
	if(last)
	{
		//phase 12,13,14,15,16,17,18
		//phase 19 RUN idle
		bscan_inst(JSTART,NULL,NULL,14,1);
	}
	return 0;
}

/***************************************************************************/
/* spa2_select_user
/***************************************************************************/
static int spi_opened = 0;
DWORD spa2_select_user(BYTE port)
{
	int ret;
	BYTE cmd;

	// close now open
	if(spi_opened)
	{
		ret = bscan_dr(NULL,NULL,6,1);
		if(!ret)
			ret = bscan_reset(2);
		spi_opened = 0;
	}

	cmd = BYPASS;
	switch(port)
	{
	case 1: cmd = USER1; break;
	case 2: cmd = USER2; break;
	}
	/* enter USER1 or USER2 */
	if(!ret && cmd != BYPASS)
	{
		ret = bscan_inst(USER1,NULL,NULL,0,0);
		if(!ret)
			spi_opened = port;
	}
	return ret;
}

/***************************************************************************/
/* spa2_xfer_user
/***************************************************************************/
DWORD spa2_xfer_user(BYTE *txbuf,BYTE*rxbuf,int num_bits,int do_close)
{
	bscan_dr(txbuf,rxbuf,num_bits,do_close);
	if(do_close)
		spi_opened = 0;

	// swap read byte
	if(rxbuf)
	{
		while(num_bits/8)
		{
			num_bits-=8;
			rxbuf[num_bits/8] = swap_byte(rxbuf[num_bits/8]);
		}
		return 1;
	}
}
